﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=10.6
@EndOfDesignText@
'Code module
'Subs in this code module will be accessible from all modules.
Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.
	Type TMapTileXY(fX As Long,fY As Long)
	Type TMapLatLng(fLat As Double,fLng As Double)
	Type TMapGPS(fLatLng As TMapLatLng,fBearing As Int)
	Type TMap(fCenterLatLng As TMapLatLng,fZoomLevel As Int,fCompassDirection As Double,fShowMenu As Boolean,fShowGrid As Boolean,fShowCenter As Boolean,fShowLandmark As Boolean,fShowCenterLatLng As Boolean,fShowZoom As Boolean,fShowScale As Boolean,fOfflineMode As Boolean,fShowCompass As Boolean,fShowMarkers As Boolean,fMarkers As Map,fShowGPS As Boolean,fGPS As TMapGPS)
	Type TMapBoxLatLng(fLeftTop As TMapLatLng,fRightBottom As TMapLatLng)
	Type TMapTileNumberOffset(fTile As Long,fOffset As Double)
	Type TMapPoint(fX As Double,fY As Double)
	Type TMapMarker(fLatLng As TMapLatLng,fColor As Int)
	'public constants
	Public const cTileSize As Double=256dip
	Public const cMinLat As Double=-85.0511
	Public const cMaxLat As Double=85.0511
	Public const cMinLng As Double=-180.0
	Public const cMaxLng As Double=180.0
	Public const cMinZoomLevel As Int=0
	Public const cMaxZoomLevel As Int=17
End Sub

'return a valid lat
public Sub validLat(aLat As Double) As Double
	Return Max(cMinLat,Min(aLat,cMaxLat))
End Sub

'return a valid lng
public Sub validLng(aLng As Double) As Double
	Return Max(cMinLng,Min(aLng,cMaxLng))
End Sub

'return a valid zoomLevel
public Sub validZoomLevel(aZoomLevel As Int) As Int
	Return Max(cMinZoomLevel,Min(aZoomLevel,cMaxZoomLevel))
End Sub

'return a valid compassDirection
public Sub validCompassDirection(aCompassDirection As Double) As Double
	#if b4i
		Dim d As Double=Bit.FMod(aCompassDirection, 360)
	#Else
		Dim d As Double=aCompassDirection Mod 360
	#End If
	If d<0 Then
		d=360+d
	End If
	Return Round(d)
End Sub

'return a TLatLng from lat/lng
public Sub initLatLng(aLat As Double,aLng As Double) As TMapLatLng
	Dim ll As TMapLatLng
	ll.Initialize
	ll.fLat=validLat(aLat)
	ll.fLng=validLng(aLng)
	Return ll
End Sub

'return a TBoxLatLng from Lat/Lng (left top) and Lat/Lng (Right Bottom)
Public Sub initBoxLatLng(aLat1 As Double,aLng1 As Double,aLat2 As Double,aLng2 As Double) As TMapBoxLatLng
	Dim b As TMapBoxLatLng
	b.Initialize
	b.fLeftTop=initLatLng(aLat1,aLng1)
	b.fRightBottom=initLatLng(aLat2,aLng2)
	Return b
End Sub

'return a TMapGPS from lat/lng and bearing
public Sub initGPS(aLatLng As TMapLatLng,aBearing As Int) As TMapGPS
	Dim g As TMapGPS
	g.Initialize
	g.fLatLng=aLatLng
	g.fBearing=aBearing
	Return g
End Sub

'return a tmap
public Sub initMap(aLatLng As TMapLatLng,aZoomLevel As Int,aCompassDirection As Double,aOfflineMode As Boolean,aShowMenu As Boolean,aShowGrid As Boolean,aShowCenter As Boolean,aShowLandmark As Boolean,aShowCenterLatLng As Boolean,aShowZoom As Boolean,aShowScale As Boolean,aShowCompass As Boolean,aShowMarkers As Boolean,aMarkers As Map,aShowGPS As Boolean,aGPS As TMapGPS) As TMap
	Dim mi As TMap
	mi.fCenterLatLng=aLatLng
	mi.fCompassDirection=aCompassDirection
	mi.fOfflineMode=aOfflineMode
	mi.fShowCenter=aShowCenter
	mi.fShowCenterLatLng=aShowCenterLatLng
	mi.fShowCompass=aShowCompass
	mi.fShowGrid=aShowGrid
	mi.fShowLandmark=aShowLandmark
	mi.fShowMarkers=aShowMarkers
	mi.fShowMenu=aShowMenu
	mi.fShowScale=aShowScale
	mi.fShowZoom=aShowZoom
	mi.fZoomLevel=aZoomLevel
	mi.fMarkers=aMarkers
	mi.fShowGPS=aShowGPS
	mi.fGPS=aGPS
	Return mi
End Sub

'Return a TMapPoint from X And Y
public Sub initPoint(aX As Int,aY As Int) As TMapPoint
	Dim p As TMapPoint
	p.Initialize
	p.fX=aX
	p.fY=aY
	Return p
End Sub

'return true if a Lat/lng is in BoxLatLng
public Sub LatLngInBoxLatLng(aLatLng As TMapLatLng,aBoxLatLng As TMapBoxLatLng) As Boolean
	Return (aLatLng.fLat>=Min(aBoxLatLng.fLeftTop.fLat,aBoxLatLng.fRightBottom.fLat)) And _
	       (aLatLng.fLat<=Max(aBoxLatLng.fRightBottom.fLat,aBoxLatLng.fLeftTop.fLat)) And _
		   (aLatLng.fLng>=Min(aBoxLatLng.fLeftTop.flng,aBoxLatLng.fRightBottom.fLng)) And _
		   (aLatLng.fLng<=Max(aBoxLatLng.fRightBottom.fLng,aBoxLatLng.fLeftTop.flng))
End Sub

'return a TTileXY from X and Y
public Sub initTileXY(aX As Long,aY As Long) As TMapTileXY
	Dim fTileXY As TMapTileXY
	fTileXY.Initialize
	fTileXY.fX=aX
	fTileXY.fY=aY
	Return fTileXY
End Sub

'return a marker
public Sub initMarker(aLatLng As TMapLatLng,aColor As Int) As TMapMarker
	Dim m As TMapMarker
	m.Initialize
	m.fLatLng=aLatLng
	m.fColor=aColor
	Return m
End Sub

'return X tile and offset for this lng
public Sub LngToTileX(aLng As Double,aZoom As Int) As TMapTileNumberOffset
	Dim v As Double=(aLng+180)/360*Power(2,aZoom)
	Dim t As TMapTileNumberOffset
	t.Initialize
	t.fTile=Floor(v)
	t.fOffset=(v-t.fTile)*cTileSize
	Return t
End Sub

'return Y tile and offset for this lat
public  Sub LatToTileY(aLat As Double,aZoom As Int) As TMapTileNumberOffset
	Dim v As Double=(1-Logarithm(TanD(aLat)+1/CosD(aLat),cE)/cPI)/2*Power(2,aZoom)
	Dim t As TMapTileNumberOffset
	t.Initialize
	t.fTile=Floor(v)
	t.fOffset=(v-t.fTile)*cTileSize
	Return t
End Sub

'return lng for X tile and offset
public Sub TileXToLng(aX As Double,aOffsetX As Double,aZoom As Int) As Double
	Dim n As Double=Power(2,aZoom)
	Dim v As Double=(aX+aOffsetX/cTileSize)/n*360.0-180.0
	Return v
End Sub

'return lat for Y tile and offset
public Sub TileYToLat(aY As Double,aOffsetY As Double,aZoom As Int) As Double
	Dim n As Double=cPI-2*cPI*(aY+aOffsetY/cTileSize)/Power(2,aZoom)
	Dim v As Double=(180.0/cPI*ATan(0.5*(Power(cE,n)-Power(cE,-n))))
	Return v
End Sub
